#include "lcdHeader.h"

// This program and its header file assumes the following
// order of the LCD pins from the Arduino: RS-RW-EN-D0...D7.

int rs=2; // Pin used for Register Select
int rw=rs+1; // Pin used for read-write select
int en=rs+2; // Pin used for enable
int d0=rs+3; // Pin used for Data 0 pin
int d1=rs+4; // "   "     "    "  1 pin
int d2=rs+5; // "   "     "    "  2 pin
int d3=rs+6; // "   "     "    "  3 pin
int d4=rs+7; // "   "     "    "  4 pin
int d5=rs+8; // "   "     "    "  5 pin
int d6=rs+9; // "   "     "    "  6 pin
int d7=rs+10; // "   "     "    "  7 pin
int dly=2; // Delay, in milliseconds

void setup() {
  // put your setup code here, to run once:

  // Tell the Arduino that all of the following
  // pins will be for output.
  pinMode(rs,OUTPUT);
  pinMode(rw,OUTPUT);
  pinMode(en,OUTPUT);
  pinMode(d0,OUTPUT);
  pinMode(d1,OUTPUT);
  pinMode(d2,OUTPUT);
  pinMode(d3,OUTPUT);
  pinMode(d4,OUTPUT);
  pinMode(d5,OUTPUT);
  pinMode(d6,OUTPUT);
  pinMode(d7,OUTPUT);

 // Clear the display
 
 lcdClear(rs); // This comes from the header file.
 
 // Set the display to 8-bit, 2-lines, 5x8 font
 // I call this "8x2x8".
 
 digitalWrite(rs,0); // \
 digitalWrite(rw,0); //  \
 digitalWrite(d7,0); //   > "Function Set" command
 digitalWrite(d6,0); //  /
 digitalWrite(d5,1); // /
 digitalWrite(d4,1); // Set to 8-bit instructions & data
 digitalWrite(d3,1); // Set to two lines
 digitalWrite(d2,0); // Set to 5x8 character font
 digitalWrite(d1,0); // Ignored
 digitalWrite(d0,0); // Ignored
 digitalWrite(en,1);
 delay(dly);
 digitalWrite(en,0);

 // Turn the display on, the cursor off, and set the cursor to not blinking

 digitalWrite(rs,0); // \
 digitalWrite(rw,0); //  \
 digitalWrite(d7,0); //   \
 digitalWrite(d6,0); //    > Display on/off control
 digitalWrite(d5,0); //   / 
 digitalWrite(d4,0); //  /
 digitalWrite(d3,1); // /
 digitalWrite(d2,1); // Turn the display on. Otherwise, the text will not be visible.
 digitalWrite(d1,0); // Turn the cursor off. If you want the cursor on, set it to 1.
 digitalWrite(d0,0); // Set the cursor to not blinking. If you want it to blink, set it to 1.
 digitalWrite(en,1);
 delay(dly);
 digitalWrite(en,0);

 // Set the entry mode, with the cursor incrementing and the display not shifting

 digitalWrite(rs,0); // \
 digitalWrite(rw,0); //  \
 digitalWrite(d7,0); //   \
 digitalWrite(d6,0); //    \
 digitalWrite(d5,0); //     > Entry mode set
 digitalWrite(d4,0); //    /
 digitalWrite(d3,0); //   /
 digitalWrite(d2,1); //  /
 digitalWrite(d1,1); // Increment the cursor
 digitalWrite(d0,0); // Do not shift the display
 digitalWrite(en,1);
 delay(dly);
 digitalWrite(en,0);

}

void loop() {
  // put your main code here, to run repeatedly:

 String myText="Hello, my name is Gary. I'm programming this Arduino to write text to a small LCD screen. In order for this to work, I have to figure out how to make it properly display over several lines, with proper word wrapping. We'll see how it goes."; 
 String myWord=""; // This will be the text passed to the LCD screen line-by-line.
 int startPt=0; // This will be the starting character for each bit of text pulled from the main text above.
 int endPt=0; // This will be the ending point for each bit of text pulled from the main text above.
 int lineLength=20; // This should be the length of one line on the LCD screen. This should be either 16 or 20.
 int lineNum=1; // This counts how many lines have already been written to the screen.
 int totalLines=4; // This is the total number of lines on the screen, and should be 2 or 4.

lcdClear(rs);
  while(endPt<myText.length()) {

    endPt=lastSpace(&myText[0],startPt,lineLength); // Find the space character where the screen lines can be split.
    myWord=pullText(&myText[0],startPt,endPt); // Pull out the text from the main text string based on the space.
                                               // character found on the last line.
    lcdText(myWord,lineNum,1,rs); // Write the text to the LCD.
    startPt=endPt+1; // The last, pulled character is a space. The +1 skips over the space to the next character.
    lineNum=lineNum+1; // Increment the line number
    // Check whether you've written the last line to the screen. If so, set a delay (pause),
    // clear the screen, reset the line number, and continue writing to the screen.
    if(lineNum==(totalLines+1)) {
      delay(5000);
      lcdClear(rs);
      lineNum=1;
    }
  }
  if(lineNum>1) {
    delay(5000); // Only pause if the screen is not blank.
  }

}
